/** 
	A "hello" program in Qt
	that use OpenGL drawing primitives
	File: quad.cpp

	Fraga 19/09/2001
**/
#include <qpainter.h>
#include <ctype.h>
#include "cube.h"

Cube::Cube( QWidget *parent )
        : QGLWidget( parent )
//  : QGLWidget( QGLFormat( SingleBuffer|NoDepthBuffer ), parent, name )
{
	signo = 1;	

	// setMouseTracking( TRUE );

	// setFocusPolicy( QWidget::StrongFocus);
	setFocusPolicy( Qt::StrongFocus);

	timer = new QTimer( this );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	xrot = yrot = 0;
	angle = 0;

	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
	createCube( 0.6 );
}


void Cube::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Cube::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
    xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

void Cube::keyPressEvent ( QKeyEvent * e )
{
	switch ( e->key() ) {
		case '+':   
			signo = 1;
			break;
		case '-':  
			signo = -1;
			break;
	}
}

//
// This method puts the initial coordinates each time window is resized
//
void Cube::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	if( w <= h )
		glOrtho ( -1.0, 1.0, -1.0/aspect, 1.0/aspect, -1.0, 1.0);
	else
		glOrtho (-aspect, aspect, -1.0, 1.0, -1.0, 1.0);
	
	emit changeSize ( );
}

QSizePolicy Cube::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Cube::paintGL(void)
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();             /* clear the matrix */
	glRotatef (yrot, 0.0, 1.0, 0.0);
	glRotatef (xrot, 1.0, 0.0, 0.0);


	glRotatef ( angle , 0.0, 0.0, 1.0);

	showCube ( );	

	swapBuffers( );
}

void Cube::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

	glEnable (GL_DEPTH_TEST);
}

void Cube::createCube( float l )
{
	v[0][0] = l; v[0][1] = l; v[0][2] = l;
	v[1][0] =-l; v[1][1] = l; v[1][2] = l;
	v[2][0] =-l; v[2][1] = l; v[2][2] =-l;
	v[3][0] = l; v[3][1] = l; v[3][2] =-l;
	v[4][0] = l; v[4][1] =-l; v[4][2] = l;
	v[5][0] =-l; v[5][1] =-l; v[5][2] = l;
	v[6][0] =-l; v[6][1] =-l; v[6][2] =-l;
	v[7][0] = l; v[7][1] =-l; v[7][2] =-l;

	c[0][0] = 0; c[0][1] = 1; c[0][2] = 5; c[0][3] = 4;
	c[1][0] = 2; c[1][1] = 3; c[1][2] = 7; c[1][3] = 6;
	c[2][0] = 0; c[2][1] = 4; c[2][2] = 7; c[2][3] = 3;
	c[3][0] = 1; c[3][1] = 2; c[3][2] = 6; c[3][3] = 5;
	c[4][0] = 0; c[4][1] = 3; c[4][2] = 2; c[4][3] = 1;
	c[5][0] = 4; c[5][1] = 5; c[5][2] = 6; c[5][3] = 7;


	color[0][0] = 1.0; color[0][1] = 1.0; color[0][2] = 1.0;
	color[1][0] = 1.0; color[1][1] = 0.0; color[1][2] = 0.0;
	color[2][0] = 0.0; color[2][1] = 0.0; color[2][2] = 1.0;
	color[3][0] = 0.0; color[3][1] = 1.0; color[3][2] = 1.0;
	color[4][0] = 0.0; color[4][1] = 1.0; color[4][2] = 0.0;
	color[5][0] = 1.0; color[5][1] = 1.0; color[5][2] = 0.0;
} 

void Cube::showCube(  )
{
	glBegin( GL_QUADS );
	for( int i=0; i<6; i++ ) {
		glColor3fv( &color[i][0] );
		for( int j=0; j<4; j++ ) {
			glVertex3fv( &v[ c[i][j] ][0] );
		}
	}
	glEnd(  );
}
